; ========================================================================
; Script to implement an automatic or semi-automatic adjustment of:
; Luminosity, Contrast, Saturation
; ----------------------------------------------------------------------
; initial version
;
; note: it includes a service function provided by Saul Goode (thanks!)
;       to detect the saturation level
;
; Flow implemented to get the final result:
; 
; 0.  Analyse action requested:
;
; A.  If action = Info only:
;
; A1. get from the histogram the Mean Value, the Std Deviation, the Saturation level
; A2. compare them with the targets and calculate the deltas
; A3. according to these deltas classify:
;       the current Luminosity: Low, Average, High
;       the current Contrast: Low, Average, High
;       the current Saturation: Low, Average, High
; note: Average means around the target +/- 32
; A4. display to the user the calculated classes
;
; B.  action = Auto enhance:
;
; B1. analyse the level of strength requested for the adjustments
; B2. if not "auto", all factors are equally set to the level requested
; B3. if "auto" each single factor is set according to the current class:
; B4. - strong for a low class
;     - normal for an average class
;     - strong for a high class
; B5. calculate the current Mean Value from the histogram
; B6. calculate the adjusting Set Value
; B7. apply it, using the gimp-levels native filter
; B8. calculate the current Contrast from the histogram (std deviation)
; B9. calculate the adjusting Set Value
; B10.apply it, using the gimp-brightess-contrast native filter
; B11.calculate the current Saturation from Saul Goode's service routine
; B12.calculate the adjusting Set Value
; B13.apply it, using the gimp-hue-saturation native filter 
; ----------------------------------------------------------------
; ----------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, you can view the GNU General Public
; License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
; Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
; Ave, Cambridge, MA 02139, USA.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
(define (get-saturation layer)
  (let* ((image (car (gimp-item-get-image layer)))
         (sat-image (car (plug-in-decompose RUN-NONINTERACTIVE 
                                            image 
                                            layer 
                                            "Saturation" 
                                            FALSE )))
         (sat-layer (car (gimp-image-get-active-layer sat-image)))
         (saturation (car (gimp-histogram sat-layer
                                          HISTOGRAM-VALUE
                                          0
                                          255 )))
         ) ; end of internal definitions 
    (gimp-image-delete sat-image)
    saturation
    ) ; end let
  ) ; end define
; ===================================================================================================================
;  
(define (script-fu-auto-LCS-enhance
            theImage
            theLayer
            action
            strength
            target-L
            target-C
            target-S
            )

; Initialize an undo, so the process can be undone with a single undo
 (gimp-image-undo-group-start theImage)
 (let* (
        (histoValues 0)
        (meanValue 0)
	(stdDev 0)
        (meanSat 0)
        (LowL 0)
        (AvgL 1)
        (HighL 2)
        (LowC 0)
        (AvgC 1)
        (HighC 2)
        (LowS 0)
        (AvgS 1)
        (HighS 2)
        (class (list "Low" "Average" "High"))
        (RfactorL 9)
        (RfactorC 9)
        (RfactorS 9)
        (Luminosity 9)
        (Contrast 9)
        (Saturation 9)
        (setMeanValue)
        (setContrast)
        (setSaturation)
       )
  
 ; perform requested action
 ; ------------------------
 
 (if (= action 1)
  
  (begin            ; logic for info only
                    ; ===================
    (set! histoValues (gimp-histogram theLayer HISTOGRAM-VALUE 0 255))
    (set! meanValue (car histoValues))
    ;(gimp-message (string-append "mean value: " (number->string meanValue)))
    
    (set! stdDev (car (cdr histoValues)))
    ;(gimp-message (string-append "std deviation: " (number->string stdDev)))
    
    (set! meanSat (get-saturation theLayer))  ; mean of values
    ;(gimp-message (string-append "mean saturation: " (number->string meanSat)))
    
    (if (< meanValue (- target-L 32))
        (set! Luminosity LowL)
        (if (> meanValue (+ target-L 32))
            (set! Luminosity HighL)
            (set! Luminosity AvgL)
         )
     )
    (if (< stdDev (- target-C 32))
        (set! Contrast LowC)
        (if (> stdDev (+ target-C 32))
            (set! Contrast HighC)
            (set! Contrast AvgC)
         )
     )
    (if (< meanSat (- target-S 32))
        (set! Saturation LowS)
        (if (> meanSat (+ target-S 32))
            (set! Saturation HighS)
            (set! Saturation AvgS)
         )
     )
    
    ;(gimp-message (string-append "Luminosity class: " (list-ref class Luminosity)))
    ;(gimp-message (string-append "Contrast class: " (list-ref class Contrast)))
    ;(gimp-message (string-append "Saturation class: " (list-ref class Saturation)))

  ); end begin if action = info only
 ; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
 (begin                         ; logic for auto enhance start here
                                ; =================================
; apply choosen strength

(if (> strength 0))            ; not auto
  (begin
    (if (= strength 3)         ; weak adjustment
       (set! RfactorL 3)
       (if (= strength 1)      
           (set! RfactorL 1)     ; strong adjustmnet
           (set! RfactorL 2)     ; normal adjustment
       ) ; end if 1
     ) ; end if 3
    (set! RfactorC RfactorL)    ; all 3 equal
    (set! RfactorS RfactorL)    ; all 3 equal
   ) ; end Begin  
  ;  else auto adjustment <================================================
  ; fix target
  ;(set! target-L 120)
  ;(set! target-C 80)
  ;(set! target-S 120)
  ; auto is based on current status
  (begin
    (if (= Luminosity HighL)    (set! RfactorL 3) ; strong against high     ; was 3
        (if (= Luminosity LowL) (set! RfactorL 3) ; strong against low      ; was 3
                                (set! RfactorL 2) ; normal against average
         )
     ) ; end if Luminosity
    (if (= Contrast HighC)      (set! RfactorC 3) ; strong against high     ; was 3
        (if (= Contrast LowC)   (set! RfactorC 3) ; strong against low      ; was 3
                                (set! RfactorC 2) ; normal against normal
         )
     ) ; end if Contrast
    (if (= Saturation HighS)    (set! RfactorS 3) ; strong against high     ; was 3
        (if (= Saturation LowS) (set! RfactorS 3) ; strong against low      ; was 3
                                (set! RfactorS 2) ; normal against normal
         )
     ) ; end if Saturation
)   

; mean value
    (set! histoValues (gimp-histogram theLayer HISTOGRAM-VALUE 0 255))
    (set! meanValue (car histoValues))
    ;(gimp-message (string-append "old mean value: " (number->string meanValue)))
    (set! setMeanValue (/ target-L meanValue))                       ; the ratio 
    (set! setMeanValue (/ (+ setMeanValue RfactorL) (+ RfactorL 1))) ; reduce the ratio
    (set! setMeanValue (* setMeanValue setMeanValue))                ; power of the ratio
    (if (< setMeanValue 0.1)                                         ; check range 
        (set! setMeanValue 0.1)
        (if (> setMeanValue 10)
            (set! setMeanValue 10)
         )
     )
    ;(gimp-message (string-append "set midpoint: " (number->string setMeanValue)))
    (gimp-levels theLayer HISTOGRAM-VALUE 0 255 setMeanValue 0 255)
    (set! histoValues (gimp-histogram theLayer HISTOGRAM-VALUE 0 255))
    (set! meanValue (car histoValues))
    ;(gimp-message (string-append "new mean value: " (number->string meanValue)))
    
; standard deviation
    (set! histoValues (gimp-histogram theLayer HISTOGRAM-VALUE 0 255))
    (set! stdDev (car (cdr histoValues)))
    ;(gimp-message (string-append "std deviation: " (number->string stdDev)))
    (set! setContrast (- target-C stdDev))                       ; the difference
    ;(gimp-message (string-append "target-C: " (number->string target-C)))
    (set! setContrast (/ setContrast RfactorC))                  ; reduce the difference
    (if (< setContrast -127)                                     ; check range 
        (set! setContrast -127)
        (if (> setContrast 127)
            (set! setContrast 127)
         )
     )
    ;(gimp-message (string-append "delta contrast: " (number->string setContrast)))
    (gimp-brightness-contrast theLayer 0 setContrast)
    (set! histoValues (gimp-histogram theLayer HISTOGRAM-VALUE 0 255))
    (set! stdDev (car (cdr histoValues)))
    ;(gimp-message (string-append "new std deviation: " (number->string stdDev)))
    
; mean saturation    
    (set! meanSat (get-saturation theLayer))  ; mean of values
    ;(gimp-message (string-append "mean saturation: " (number->string meanSat)))
    (set! setSaturation (- target-S meanSat))                      ; the difference
    (set! setSaturation (/ setSaturation RfactorS))           ; reduce the difference
    (set! setSaturation (* (/ setSaturation 128) 100))        ; set as percentage
    (if (< setSaturation -100)                                ; check range -100/100
        (set! setSaturation -100)
        (if (> setSaturation 100)
            (set! sSaturation 100)
         )
     )
    ;(gimp-message (string-append "delta saturation: " (number->string setSaturation)))
    (gimp-hue-saturation theLayer 0 0 0 setSaturation)
    (set! meanSat (get-saturation theLayer))  ; mean of values
    ;(gimp-message (string-append "new mean saturation: " (number->string meanSat)))
 
  ) ; end begin for enhancing
 ) ; end if action...                                
 
    (gimp-displays-flush)

 ) ; end let
 (gimp-image-undo-group-end theImage)
) ; end define

(script-fu-register "script-fu-auto-LCS-enhance"
            _"<Image>/DNscripts/Colors/Auto Luminosity-Contrast-Saturation enhance"
            "filter to auto improve luminosity, contrast and saturation"
            "Diego Nassetti"
            "Diego Nassetti"
            "05/01/2015"
            "RGB* "
            SF-IMAGE        "Image"     0
            SF-DRAWABLE     "Drawable"  0
            SF-OPTION	    "Action requested" '("auto enhance" "info only")
            SF-OPTION	    "Strength of adjustment" '("auto" "strong" "normal" "weak")
            SF-ADJUSTMENT   "Target luminosity"      '(120 32 224 8 16 0 0)
            SF-ADJUSTMENT   "Target contrast"        '(80 32 224 8 16 0 0)
            SF-ADJUSTMENT   "Target saturation"      '(120 32 224 8 16 0 0)
)